"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const meta_1 = require("../meta");
const path = require('path');
const glob = require('glob');
const recommended = require('./recommended');
function deepMerge(dst, src) {
    for (const key of Object.keys(src)) {
        if (dst.hasOwnProperty(key) && typeof dst[key] === 'object' && typeof src[key] === 'object') {
            deepMerge(dst[key], src[key]);
        }
        else {
            dst[key] = src[key];
        }
    }
    return dst;
}
const parseSeverityLut = [
    'disable',
    'warn',
    'error',
];
function parseSeverity(value) {
    if (typeof value === 'number') {
        return value;
    }
    else {
        return parseSeverityLut.indexOf(value.toLowerCase());
    }
}
class Config {
    constructor(options) {
        this.loadDefaults();
        this.mergeInternal(options || {});
        this.metaTable = null;
        const self = this;
        this.config.extends.forEach(function (ref) {
            const base = Config.fromFile(ref);
            self.config = base.mergeInternal(self.config);
        });
    }
    static empty() {
        return new Config();
    }
    static fromObject(options) {
        return new Config(options);
    }
    static fromFile(filename) {
        if (filename === 'htmlvalidate:recommended') {
            return Config.fromObject(recommended);
        }
        const json = require(filename);
        json.extends = (json.extends || []).map(function (ref) {
            return Config.expandRelative(ref, path.dirname(filename));
        });
        return new Config(json);
    }
    merge(rhs) {
        return new Config(this.mergeInternal(rhs.config));
    }
    loadDefaults() {
        this.config = {
            extends: [],
            rules: {},
        };
    }
    getMetaTable() {
        if (!this.metaTable) {
            this.metaTable = new meta_1.MetaTable();
            const root = path.resolve(__dirname, '..', '..');
            for (const filename of glob.sync(`${root}/elements/*.json`)) {
                this.metaTable.loadFromFile(filename);
            }
        }
        return this.metaTable;
    }
    static expandRelative(src, currentPath) {
        if (src[0] === '.') {
            return path.normalize(`${currentPath}/${src}`);
        }
        return src;
    }
    mergeInternal(config) {
        deepMerge(this.config, config);
        return this.config;
    }
    get() {
        return Object.assign({}, this.config);
    }
    getRules() {
        const rules = Object.assign({}, this.config.rules || {});
        for (const name in rules) {
            let options = rules[name];
            if (!Array.isArray(options)) {
                options = [options, {}];
            }
            else if (options.length === 1) {
                options.push({});
            }
            options[0] = parseSeverity(options[0]);
            rules[name] = options;
        }
        return rules;
    }
}
Config.SEVERITY_DISABLED = 0;
Config.SEVERITY_WARN = 1;
Config.SEVERITY_ERROR = 2;
exports.Config = Config;
