"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require('path');
const config_1 = require("./config");
describe('config', function () {
    const expect = require('chai').expect;
    it('should load defaults', function () {
        const config = config_1.Config.empty();
        expect(config.get()).to.not.be.undefined;
    });
    it('should contain no rules by default', function () {
        const config = config_1.Config.empty();
        expect(Object.keys(config.get().rules)).to.have.lengthOf(0);
    });
    describe('getRules()', function () {
        it('should return parsed rules', function () {
            const config = config_1.Config.fromObject({ rules: { foo: 'error' } });
            expect(config.get().rules).to.deep.equal({ foo: 'error' });
            expect(config.getRules()).to.deep.equal({
                foo: [config_1.Config.SEVERITY_ERROR, {}],
            });
        });
        it('getRules() should parse severity from string', function () {
            const config = config_1.Config.fromObject({
                rules: {
                    foo: 'error',
                    bar: 'warn',
                    baz: 'disable',
                },
            });
            expect(config.getRules()).to.deep.equal({
                foo: [config_1.Config.SEVERITY_ERROR, {}],
                bar: [config_1.Config.SEVERITY_WARN, {}],
                baz: [config_1.Config.SEVERITY_DISABLED, {}],
            });
        });
        it('getRules() should retain severity from integer', function () {
            const config = config_1.Config.fromObject({
                rules: {
                    foo: 2,
                    bar: 1,
                    baz: 0,
                },
            });
            expect(config.getRules()).to.deep.equal({
                foo: [config_1.Config.SEVERITY_ERROR, {}],
                bar: [config_1.Config.SEVERITY_WARN, {}],
                baz: [config_1.Config.SEVERITY_DISABLED, {}],
            });
        });
        it('getRules() should retain options', function () {
            const config = config_1.Config.fromObject({
                rules: {
                    foo: [2, { foo: true }],
                    bar: ["error", { bar: false }],
                },
            });
            expect(config.getRules()).to.deep.equal({
                foo: [config_1.Config.SEVERITY_ERROR, { foo: true }],
                bar: [config_1.Config.SEVERITY_ERROR, { bar: false }],
            });
        });
    });
    describe('fromFile()', function () {
        it('should support JSON', function () {
            const config = config_1.Config.fromFile(`${process.cwd()}/test-files/config.json`);
            expect(config.getRules()).to.deep.equal({
                foo: [config_1.Config.SEVERITY_ERROR, {}],
                bar: [config_1.Config.SEVERITY_WARN, {}],
                baz: [config_1.Config.SEVERITY_DISABLED, {}],
            });
        });
    });
    describe('extend', function () {
        it('should extend base configuration', function () {
            const config = config_1.Config.fromObject({
                extends: [`${process.cwd()}/test-files/config.json`],
                rules: {
                    foo: 1,
                },
            });
            expect(config.getRules()).to.deep.equal({
                foo: [config_1.Config.SEVERITY_WARN, {}],
                bar: [config_1.Config.SEVERITY_WARN, {}],
                baz: [config_1.Config.SEVERITY_DISABLED, {}],
            });
        });
        it('should support deep extending', function () {
            const config = config_1.Config.fromObject({
                extends: [`${process.cwd()}/test-files/config-extending.json`],
            });
            expect(config.getRules()).to.deep.equal({
                foo: [config_1.Config.SEVERITY_ERROR, {}],
                bar: [config_1.Config.SEVERITY_WARN, {}],
                baz: [config_1.Config.SEVERITY_ERROR, {}],
            });
        });
        it('should support htmlvalidate:recommended', function () {
            const config = config_1.Config.fromObject({
                extends: ['htmlvalidate:recommended'],
            });
            expect(config.getRules()).to.be.an('object');
        });
    });
    describe('expandRelative()', function () {
        it('should expand ./foo', function () {
            expect(config_1.Config.expandRelative('./foo', '/path')).to.equal(path.join(path.sep, 'path', 'foo'));
        });
        it('should expand ../foo', function () {
            expect(config_1.Config.expandRelative('../foo', '/path/bar')).to.equal(path.join(path.sep, 'path', 'foo'));
        });
        it('should not expand /foo', function () {
            expect(config_1.Config.expandRelative('/foo', '/path')).to.equal('/foo');
        });
        it('should not expand foo', function () {
            expect(config_1.Config.expandRelative('foo', '/path')).to.equal('foo');
        });
    });
    describe('getMetaTable()', function () {
        it('should load metadata', function () {
            const config = config_1.Config.empty();
            const metatable = config.getMetaTable();
            expect(Object.keys(metatable.elements)).not.to.have.lengthOf(0);
        });
    });
});
